@@ -9,7 +9,7 @@ fi
 
 grep "slocate\|updatedb" $CRONTAB >/dev/null 2>&1
 
-if [ $? == 1 ] ; then
+if [ "$?" -eq "1" ] ; then
 
    # Check if updatedb exists
    if [ -x /usr/bin/updatedb ] ; then
