@@ -1,9 +1,9 @@
 /*****************************************************************************
- * Secure Locate v2.5                                                        *
- * December 30, 2000                                                         *
+ * Secure Locate v2.6                                                        *
+ * May 21, 2001                                                         *
  * Programmed by: Kevin Lindsay                                              *
- * Copyright (c) 1999, 2000 Kevin Lindsay & Netnation Communications Inc. &  *
- *                          James A. Woods <jwoods@adobe.com>                *
+ * Copyright (c) 1999, 2000, 2001 Kevin Lindsay & Netnation Communications   *
+ *                          Inc. & James A. Woods <jwoods@adobe.com>         *
  *                                                                           *
  * NetNation Communications   http://www.netnation.com/                      *
  * Secure Locate: ftp://ftp.geekreview.org/slocate/                          *
@@ -37,6 +37,8 @@
  *           Hans-Juergen Godau <godau@wi-inf.uni-essen.de>                  *
  *                * Fixed a segfault when searching through more than one    *
  *                  database.                                                *
+ *           Francis Liu <fxl@enstor.com.au>                                 *
+ *                * Solaris Patch                                            *
  *                                                                           *
  * Report any Bugs to: klindsay@mkintraweb.com                               *
  *                                                                           *
@@ -101,7 +103,7 @@
 
 /* GLOBALS */
 
-#define SL_RELEASE "December 30, 2000"
+#define SL_RELEASE "May 21, 2001"
 #define SL_VERSION "Secure Locate " VERSION " - Released " SL_RELEASE "\n"
 #define GRPFILE "/etc/group"
 #define SLOC_ESC -0x80
@@ -124,13 +126,19 @@ char **SLOCATE_PATH = NULL;
 char *SLOCATEDB = "/var/db/slocate/slocate.db";
 char *TMPSLOCATEDB = "/var/db/slocate/slocate.db.tmp";
 char *SLOCATEDB_DIR = "/var/db/slocate/";
+#elif defined(__SunOS__)
+char *SLOCATEDB = "/var/db/slocate/slocate.db";
+char *TMPSLOCATEDB = "/var/db/slocate/slocate.db.tmp";
+char *SLOCATEDB_DIR = "/var/db/slocate/";
+#undef MTAB_FILE
+#define MTAB_FILE "/etc/mnttab"
 #else
-/* These paths are good for Linux machines */
 char *SLOCATEDB = "/var/lib/slocate/slocate.db";
 char *TMPSLOCATEDB = "/var/lib/slocate/slocate.db.tmp";
 char *SLOCATEDB_DIR = "/var/lib/slocate/";
-# define UPDATEDB_CONF "/etc/updatedb.conf"
 #endif
+
+# define UPDATEDB_CONF "/etc/updatedb.conf"
 char *EXCLUDE_DIR=NULL;
 int EXCLUDE=0;
 int VERBOSE=0;
@@ -149,7 +157,11 @@ uid_t UID;
 gid_t GID;
 int first=1;
 char slevel='1';
+#ifdef __SunOS__
+unsigned short SLOC_GID=60001; /* Seems to be the default nobody gid */
+#else
 unsigned short SLOC_GID=65534; /* Seems to be the default nobody gid */
+#endif
 int max_queries=0;
 int ADD_SLOCATEDB = 1;
 
@@ -165,7 +177,7 @@ void
 {
 	int i;
 	printf("%s\n"
-	       "Copyright (c) 1999, 2000 Kevin Lindsay & Netnation Communications Inc. &\n"
+	       "Copyright (c) 1999, 2000, 2001 Kevin Lindsay & Netnation Communications Inc. &\n"
 	       "James A. Woods <jwoods@adobe.com>\n\n"
 	       "search usage:   %s [-qi] [-d <path>] [--database=<path>] <search string>...\n"
 	       "                %s [-r <regexp>] [--regexp=<regexp>]\n"
@@ -208,7 +220,7 @@ void
 	       "   --regexp=<regexp>  - Search the database using a basic POSIX regular\n"
 	       "                        expression.\n"
 	       "   -o <file>\n"
-	       "   --output=<file>    - Specfies the database to create.\n"
+	       "   --output=<file>    - Specifies the database to create.\n"
 	       "   -d <path>\n"
 	       "   --database=<path>  - Specfies the path of databases to search in.\n"
 	       "   -h\n"
@@ -327,7 +339,7 @@ void
 	char *part;
 	int i;
 	int res_errno;
-	
+
 	/* Make sure path is not empty */
 	if (!path || strlen(path) == 0) return;
 	
@@ -349,7 +361,7 @@ void
 	
 	/* Get first path just incase a path with ':'s were specified*/
 	part = strtok(pathcopy, ":");
-	
+
 	i = 0;
 	/* Loop through all paths and validate them.
 	 * use SLOCATE_PATH as an array of pointers to each
@@ -357,7 +369,7 @@ void
 	while (part) {
 		/* Make sure the path is valid */
 		if (!(res_errno = validate_db(part)))
-		    SLOCATE_PATH[i++] = part;
+		  SLOCATE_PATH[i++] = part;
 		else {
 			if (res_errno == -1)
 			    report_error(WARNING,QUIET,"%s: this is not a valid slocate database: %s\n",progname,part);
@@ -417,7 +429,7 @@ void
 
 		*regexp = 0;
 		strcat(regexp,ptr);
-		while ((database = SLOCATE_PATH[i++])) res |= decode_db(database, ptr);
+		while (SLOCATE_PATH && (database = SLOCATE_PATH[i++])) res |= decode_db(database, ptr);
 		exit(!res);
 	}
 	
@@ -609,12 +621,12 @@ int
 	
 #ifdef __FreeBSD__
 	{
-		struct statfs fs_stat;
+		struct statfs *fs_stat;
 		long bufsize=4096;
 		int num_mounts=0;
 		int i;
 		char *exclude_str=NULL;
-		
+
 		fs_stat = malloc(sizeof(struct statfs));
 		if (!fs_stat)
 		    report_error(FATAL,QUIET,"%s: parse_fs_exclude: 'fs_stat': malloc: %s\n",progname,strerror(errno));
@@ -954,7 +966,8 @@ int
 	
 	if (first) {
 		chmod(TMPSLOCATEDB,00640);
-		chown(TMPSLOCATEDB,UID,SLOC_GID);
+		if (UID == 0)
+		    chown(TMPSLOCATEDB,UID,SLOC_GID);
 		first=0;
 	}
 	
@@ -1058,6 +1071,44 @@ int
 	return res;
 }
 
+/* Check to make sure the entire path is readable */
+int
+    check_path_access(char *codedpath)
+{
+	char *dir = NULL;
+	char *path = NULL;
+	int res;
+	int len = 0;
+	char *str_ptr;
+
+	if (access(codedpath, R_OK) != 0) {
+		free(codedpath);
+		return 0;
+	}
+	
+	path = malloc(strlen(codedpath)+1);
+	*path = 0;
+	
+	res = 1;
+	str_ptr = codedpath;
+	
+	while (dir = strtok(str_ptr, "/")) {
+		strcat(path,"/");
+		strcat(path,dir);
+		if (access(path, R_OK) != 0) {
+			res = 0;
+			break;
+		}
+		str_ptr = NULL;
+	}
+
+	free(codedpath);
+
+        free(path);
+	
+        return res;
+}
+
 /* Decode Database */
 
 int
@@ -1101,7 +1152,7 @@ int
 	char *cp=NULL;
 #endif
 	char *bucket_of_holding=NULL;
-	
+
 	if ((fd = open(database,O_RDONLY)) == -1) {
 		report_error(WARNING,QUIET,"%s: decode_db(): %s: %s\n",progname,database,strerror(errno));
 		return(0);
@@ -1128,12 +1179,14 @@ int
 	*codedpath = 0;
 	code_ptr = codedpath;
 
+	
 	if ((globptr1 = strchr(str,'*')) != NULL ||
 	    (globptr1 = strchr(str,'?')) != NULL ||
 	    ((globptr1 = strchr(str,'[')) != NULL && strchr(str,']') != NULL))
 	    globflag = 1;
 	
-	if (REGEXP) {   
+	if (REGEXP) {
+
 		preg = malloc(sizeof(regex_t));
 		if (!preg)
 		    report_error(FATAL,QUIET,"%s: decode_db: 'preg': malloc: %s\n",progname,strerror(errno));
@@ -1223,24 +1276,24 @@ int
 			}
 			
 		}
-		
+
 //		printf("%s\n",codedpath);
 
 		if (first) {
 			code_ptr = code_ptr+strlen(codedpath);
 			first=0;
 		}
-		
+
 		pathlen--;
-		
+
 		printit=0;
-		
-		if (REGEXP)
-		    foundit = !regexec(preg,codedpath,nmatch,pmatch,0);         
-		else if (NOCASE) {
+
+		if (REGEXP) {
+		    foundit = !regexec(preg,codedpath,nmatch,pmatch,0);
+		} else if (NOCASE) {
 #ifdef FNM_CASEFOLD /* i suppose i also have strcasestr */
 			if (globflag)
-			    foundit =! fnmatch(str,codedpath,FNM_CASEFOLD);				
+			    foundit =! fnmatch(str,codedpath,FNM_CASEFOLD);
 			else
 			    foundit = (strcasestr(codedpath,str) != NULL);			
 #else /* FNM_CASEFOLD */
@@ -1266,7 +1319,7 @@ int
 		
 		if (foundit) {
 			if (slevel == '1') {
-				if (UID == 0 ||  access(codedpath, F_OK) == 0) {
+				if (UID == 0 || check_path_access(strdup(codedpath))) {
 					printit = 1;
 				}
 			} else
@@ -1300,7 +1353,7 @@ int
 int
     main(int args, char **argv)
 {
-	int res=0;
+        int res=0;
 	int ch;
 	extern char *optarg;
 	extern int optind, opterr, optopt;
@@ -1310,7 +1363,9 @@ int
 	char *spec_dir=NULL;
 	char *database;
 	int i=0;
-	
+	int o_OPT_READY = 0;
+	char *regexp_opt = NULL;
+
 	/* Get the name of the program as it was executed */
 	progname = ((p = strrchr(argv[0],'/')) ? p+1 : *argv);
 	
@@ -1390,6 +1445,7 @@ int
 			/* Check to make sure default database directory exists */
 			check_dir(SLOCATEDB_DIR);
 			ROOTDIR=1;
+			o_OPT_READY = 1;
 			ADD_SLOCATEDB = 0;
 			break;
 		 /* Update a specified database */
@@ -1397,6 +1453,7 @@ int
 			ADD_SLOCATEDB = 0;
 			ROOTDIR=0;
 			SPECDIR=1;
+			o_OPT_READY = 1;
 			spec_dir = malloc(strlen(optarg)+1);
 			if (!spec_dir)
 			    report_error(FATAL,QUIET,"%s: main: 'spec_dir': malloc: %s\n",progname,strerror(errno));
@@ -1411,18 +1468,26 @@ int
 			    report_error(FATAL,QUIET,"%s: main: 'regexp': malloc: %s\n",progname,strerror(errno));
 			*regexp = 0;
 			strcat(regexp,optarg);
-			while ((database = SLOCATE_PATH[i++])) res |= decode_db(database, optarg);
-			return(res);
-			break;
-		 /* Specify the database path to write to when creating or updating a database */
-		 case 'o':			
-			parse_create_path(optarg);
-			NEWOUTPUT=1;
+			regexp_opt = malloc(strlen(optarg)+1);
+			strcpy(regexp_opt,optarg);
+/*			while (SLOCATE_PATH && (database = SLOCATE_PATH[i++]))
+				res |= decode_db(database, optarg);
+			return(res); */
 			break;
 		 /* Specify the database to search in */
 		 case 'd':
-			parse_decode_path(optarg);
-			ADD_SLOCATEDB = 0;
+			if (args > 3) {
+				parse_decode_path(optarg);
+				ADD_SLOCATEDB = 0;
+			}
+
+			break;
+		 /* Specify the database path to write to when creating or updating a database */
+		 case 'o':
+			if (!o_OPT_READY)
+			  report_error(FATAL,QUIET,"%s: Must specify an 'Update' database option first.\n",progname);
+			parse_create_path(optarg);
+			NEWOUTPUT=1;
 			break;
 		 /* Parse --<argument> type arguments */
 		 case '-':
@@ -1467,16 +1532,20 @@ int
 	else if (ROOTDIR)
 	    res = create_db((char *)NULL);
 	/* search the database(s) */
-	else if (argv[optind]) {
+	else if (argv[optind] || REGEXP) {
 		int j = optind;
 		int i = 0;
 		
+		if (REGEXP)
+		    while (SLOCATE_PATH && (database = SLOCATE_PATH[i++]))
+			res |= decode_db(database, regexp_opt);
+
 		while (j < args) {
 			/* while ((database = SLOCATE_PATH[i++])) res |= decode_db(database, argv[j++]);
 			 i = 0;
 			 * A Bug fix by Hans-Juergen Godau <godau@wi-inf.uni-essen.de>
 			 * Prevents segfault when using multiple databases */
-			while ((database = SLOCATE_PATH[i++]))
+			while (SLOCATE_PATH && (database = SLOCATE_PATH[i++]))
 			    res |= decode_db(database, argv[j]);
 			i = 0; j += 1;
 			
