@@ -127,7 +127,7 @@ report_error(int STATUS, int QUIET, const char *format, ...) {
 	
 	/* Guess we need no more than 100 bytes. */
 	int n;
-	int size = ALLOC_SIZE;
+	unsigned int size = ALLOC_SIZE;
 	char *str;
 	va_list ap;
 	
@@ -142,11 +142,11 @@ report_error(int STATUS, int QUIET, const char *format, ...) {
 		fprintf(stderr,"report_fatal_error: malloc: %s\n",strerror(errno));
 		exit(1);
 	}
-
+	
 	while (1) {
 		/* Try to print in the allocated space. */
 		va_start(ap, format);
-		n = vsnprintf (str, size, format, ap);
+		n = vsnprintf(str, size, format, ap);
 		va_end(ap);
 
 		/* If that worked, print message. */
@@ -160,6 +160,7 @@ report_error(int STATUS, int QUIET, const char *format, ...) {
 			fflush(stderr);
 			break;
 		}
+
 		/* Else try again with more space. */
 		if (n > -1)    /* glibc 2.1 */
 		    size = n+1; /* precisely what is needed */
